@@ -1,7 +1,7 @@
 package Nodejs::Util;
 
-our $DATE = '2016-07-03'; # DATE
-our $VERSION = '0.006'; # VERSION
+our $DATE = '2018-01-03'; # DATE
+our $VERSION = '0.007'; # VERSION
 
 use 5.010001;
 use strict;
@@ -12,6 +12,7 @@ our @EXPORT_OK = qw(
                        get_nodejs_path
                        nodejs_available
                        system_nodejs
+                       nodejs_module_path
                );
 
 our %SPEC;
@@ -184,6 +185,87 @@ sub system_nodejs {
     );
 }
 
+sub nodejs_module_path {
+    my $opts = ref $_[0] eq 'HASH' ? shift : {};
+    my $module = shift;
+
+    my ($dir, $name, $ext) = $module =~ m!\A(?:(.*)/)?(.+?)(\.\w+)?\z!;
+    #use DD; dd {dir=>$dir, name=>$name, ext=>$ext};
+
+    my  @dirs;
+    if (defined $dir) {
+        @dirs = ($dir);
+    } else {
+        my $cwd = do {
+            if (defined $opts->{cwd}) {
+                $opts->{cwd};
+            } else {
+                require Cwd;
+                Cwd::getcwd();
+            }
+        };
+        $cwd =~ s!/node_modules\z!!;
+        while (1) {
+            push @dirs, "$cwd/node_modules";
+            $cwd =~ s!(.*)/.+!$1!
+                or last;
+        }
+    }
+
+    if (defined $ENV{NODE_PATH}) {
+        my $sep = $^O =~ /win32/i ? qr/;/ : qr/:/;
+        push @dirs, split($sep, $ENV{NODE_PATH});
+    }
+
+    if (defined $ENV{HOME}) {
+        push @dirs, "$ENV{HOME}/.node_modules";
+        push @dirs, "$ENV{HOME}/.node_libraries";
+    }
+
+    if (defined $ENV{PREFIX}) {
+        push @dirs, "$ENV{PREFIX}/lib/node";
+    }
+
+    #use DD; dd \@dirs;
+
+    my @res;
+    for my $d (@dirs) {
+        next unless -d $d;
+        if (defined $ext) {
+            my $p = "$d/$name$ext";
+            if (-f $p) {
+                push @res, $p;
+                last unless $opts->{all};
+            }
+        } else {
+            my $p;
+            for my $e (".js", ".json", ".node") {
+                $p = "$d/$name$e";
+                if (-f $p) {
+                    push @res, $p;
+                    last unless $opts->{all};
+                }
+            }
+            $p = "$d/$name";
+            if (-d $p) {
+                if (-f "$p/index.js") {
+                    push @res, "$p/index.js";
+                    last unless $opts->{all};
+                } elsif (-f "$p/package.json") {
+                    push @res, "$p/package.json";
+                    last unless $opts->{all};
+                }
+            }
+        }
+    }
+
+    if ($opts->{all}) {
+        return \@res;
+    } else {
+        return $res[0];
+    }
+}
+
 1;
 # ABSTRACT: Utilities related to Node.js
 
@@ -199,12 +281,16 @@ Nodejs::Util - Utilities related to Node.js
 
 =head1 VERSION
 
-This document describes version 0.006 of Nodejs::Util (from Perl distribution Nodejs-Util), released on 2016-07-03.
+This document describes version 0.007 of Nodejs::Util (from Perl distribution Nodejs-Util), released on 2018-01-03.
 
 =head1 FUNCTIONS
 
 
-=head2 get_nodejs_path(%args) -> any
+=head2 get_nodejs_path
+
+Usage:
+
+ get_nodejs_path(%args) -> any
 
 Check the availability of Node.js executable in PATH.
 
@@ -228,7 +314,11 @@ If this option is set to true, will return an array of paths intead of path.
 Return value:  (any)
 
 
-=head2 nodejs_available(%args) -> [status, msg, result, meta]
+=head2 nodejs_available
+
+Usage:
+
+ nodejs_available(%args) -> [status, msg, result, meta]
 
 Check the availability of Node.js.
 
@@ -275,7 +365,11 @@ that contains extra information.
 
 Return value:  (any)
 
-=head2 system_nodejs([ \%opts ], @argv)
+=head2 system_nodejs
+
+Usage:
+
+ system_nodejs([ \%opts ], @argv)
 
 Will call L<IPC::System::Options>'s system(), but with node.js binary as the
 first argument. Known options:
@@ -297,6 +391,37 @@ Will be passed to C<nodejs_available()>.
 
 Other options will be passed to C<IPC::System::Options>'s C<system()>.
 
+=head2 nodejs_module_path
+
+Usage:
+
+ nodejs_module_path([ \%opts, ] $module)
+
+Search module in filesystem according to Node.js rule described in
+L<https://nodejs.org/api/modules.html>. C<$module> can be either a
+relative/absolute path (e.g. C<./bip39.js>, C<../bip39.js>, or
+C</home/foo/bip39.js>), a filename (e.g. C<bip39.js>), or a filename with the
+C<.js> removed (e.g. C<bip39>).
+
+Known options:
+
+=over
+
+=item * parse_package_json => bool (default: 0)
+
+Not yet implemented.
+
+=item * cwd => str
+
+Use this directory instead of using C<Cwd::get_cwd()>.
+
+=item * all => bool
+
+If set to true, will return an array of all found paths instead of the first
+found path.
+
+=back
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Nodejs-Util>.
@@ -319,7 +444,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2016 by perlancar@cpan.org.
+This software is copyright (c) 2018, 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
